"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMakeStyle = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
require("./tools/Object.fromEntries");
const objectKeys_1 = require("./tools/objectKeys");
const react_1 = require("react");
const classnames_1 = require("./tools/classnames");
/*
import createCache from "@emotion/cache";

const key = 'custom'
export const cache = createCache({ key });
*/
/**
 * https://github.com/garronej/tss-react
 * */
function createMakeStyle(params) {
    const { useTheme, CacheProvider, serializeStyles, insertStyles, cache, getRegisteredStyles } = params;
    const { TssProviderForSsr, useCssAndCx } = createUseCssAndCx({
        CacheProvider,
        serializeStyles,
        insertStyles,
        cache,
        getRegisteredStyles
    });
    function makeStyles() {
        return function (getCssObjects) {
            function useStyles(params) {
                const theme = useTheme();
                const { css, cx } = useCssAndCx();
                const cssObjects = typeof getCssObjects === "function" ?
                    getCssObjects(theme, params) :
                    getCssObjects;
                const classes = Object.fromEntries(objectKeys_1.objectKeys(cssObjects)
                    .map(key => [key, css(cssObjects[key])]));
                return {
                    classes,
                    theme,
                    css,
                    cx
                };
            }
            //TODO: Return for composition as well.
            return { useStyles };
        };
    }
    return { makeStyles, useCssAndCx, TssProviderForSsr };
}
exports.createMakeStyle = createMakeStyle;
const { createUseCssAndCx } = (() => {
    const isWrappedIntoCacheProviderContext = react_1.createContext(false);
    const isBrowser = typeof document !== "undefined";
    function createUseCssAndCx(params) {
        const { CacheProvider, serializeStyles, insertStyles, getRegisteredStyles, cache } = params;
        const css = (...args) => {
            const serialized = serializeStyles(args, cache.registered);
            insertStyles(cache, serialized, false);
            return `${cache.key}-${serialized.name}`;
        };
        const { cx } = (() => {
            function merge(registered, css, className) {
                const registeredStyles = [];
                const rawClassName = getRegisteredStyles(registered, registeredStyles, className);
                if (registeredStyles.length < 2) {
                    return className;
                }
                return rawClassName + css(registeredStyles);
            }
            const cx = (...args) => merge(cache.registered, css, classnames_1.classnames(args));
            return { cx };
        })();
        function TssProviderForSsr(props) {
            const { children } = props;
            return (jsx_runtime_1.jsx(CacheProvider, Object.assign({ value: cache }, { children: jsx_runtime_1.jsx(isWrappedIntoCacheProviderContext.Provider, Object.assign({ value: true }, { children: children }), void 0) }), void 0));
        }
        function useCssAndCx() {
            const isWrappedIntoCacheProvider = react_1.useContext(isWrappedIntoCacheProviderContext);
            if (!isBrowser && !isWrappedIntoCacheProvider) {
                throw new Error([
                    "For SSR to work your components must",
                    `be wrapped within <${TssProviderForSsr.name}/>`
                ].join(" "));
            }
            return { css, cx };
        }
        return { TssProviderForSsr, useCssAndCx };
    }
    return { createUseCssAndCx };
})();
//# sourceMappingURL=createMakeStyle.js.map